@@ -63,6 +63,7 @@ class MusicApplet:
 
         self.__applet = applet
         self.__all = None
+        self.__constrain_size = -2      # cache to limit redundant constrains
 
         self.__prefs_dialog = None
         self.__plugins_dialog = None
@@ -166,6 +167,7 @@ class MusicApplet:
 
         self.__change_orient_cb (applet, applet.get_orient ())
 
+        applet.connect ("size-request", self.__size_request_cb)
         applet.connect ("change-background", self.__change_background_cb)
         applet.connect ("change-orient", self.__change_orient_cb)
         applet.show ()
@@ -186,10 +188,12 @@ class MusicApplet:
 
     def __query_tooltip_cb (self, widget, x, y, kbd, tooltip):
         # Yes, a new Guts each time, since GTK destroys it along with the tooltip itself.
-        guts = SongTooltipGuts ()
-        guts.show_all ()
-        guts.set_plugin (self.__plugin)
-        tooltip.set_custom (guts)
+        # First make sure the old one disconnects its handlers.
+        self.__song_tip_guts.set_plugin (None)
+        self.__song_tip_guts = SongTooltipGuts ()
+        self.__song_tip_guts.show_all ()
+        self.__song_tip_guts.set_plugin (self.__plugin)
+        tooltip.set_custom (self.__song_tip_guts)
         return True
 
     def __set_plugin (self, plugin):
@@ -201,8 +205,8 @@ class MusicApplet:
             self.__plugin_handlers = [
                 plugin.connect ("notify::playing", self.__playing_changed_cb),
                 plugin.connect ("notify::elapsed", self.__elapsed_changed_cb),
-                plugin.connect ("notify::min_rating", self.__min_rating_changed_cb),
-                plugin.connect ("notify::max_rating", self.__max_rating_changed_cb),
+                plugin.connect ("notify::min-rating", self.__min_rating_changed_cb),
+                plugin.connect ("notify::max-rating", self.__max_rating_changed_cb),
                 plugin.connect ("notify::rating", self.__rating_changed_cb),
             ]
             self.__playing_changed_cb (plugin, None)
@@ -246,7 +250,12 @@ class MusicApplet:
         if conf_value is None:
             conf_value = self.__conf.client.get_bool (full_key)
         conf_value = self.__assure_not_invisible (full_key, conf_value)
-        self.__scroller.props.visible = self.__plugin is not None and conf_value
+        if self.__plugin is not None and conf_value:
+            self.__scroller.props.visible = True
+            self.__applet.set_applet_flags (gnomeapplet.EXPAND_MINOR | gnomeapplet.EXPAND_MAJOR | gnomeapplet.HAS_HANDLE)
+        else:
+            self.__scroller.props.visible = False
+            self.__applet.set_applet_flags (gnomeapplet.EXPAND_MINOR)
 
 
     def __update_rating_visibility (self, conf_value=None):
@@ -320,13 +329,13 @@ class MusicApplet:
             self.__all = gtk.VBox ()
         self.__make_transparent (self.__all)
 
-        self.__all.pack_start (self.__scroller, padding=3)
-        self.__all.pack_start (self.__rating)
-        self.__all.pack_start (self.__time, padding=3)
-        self.__all.pack_start (self.__previous)
-        self.__all.pack_start (self.__play)
-        self.__all.pack_start (self.__next)
-        self.__all.pack_start (self.__launch)
+        self.__all.pack_start (self.__scroller, padding=3, expand=True, fill=True)
+        self.__all.pack_start (self.__rating, expand=False, fill=False)
+        self.__all.pack_start (self.__time, padding=3, expand=False, fill=False)
+        self.__all.pack_start (self.__previous, expand=False, fill=False)
+        self.__all.pack_start (self.__play, expand=False, fill=False)
+        self.__all.pack_start (self.__next, expand=False, fill=False)
+        self.__all.pack_start (self.__launch, expand=False, fill=False)
 
         if orient == gnomeapplet.ORIENT_RIGHT:
             self.__time.set_angle (90)
@@ -344,6 +353,28 @@ class MusicApplet:
         self.__all.show ()
 
 
+    def __size_request_cb (self, applet, requisition):
+        """
+        Set hints for the applet size if the song scroller is visible, to
+        prevent the applet from squeezing out other applets.
+        """
+
+        if self.__scroller.get_property ("visible"):
+            orient = self.__applet.get_orient ()
+            request = self.__scroller.size_request ()
+            if orient == gnomeapplet.ORIENT_UP or orient == gnomeapplet.ORIENT_DOWN:
+                min_size = requisition.width - request[0]
+            else:
+                min_size = requisition.height - request[1]
+            if self.__constrain_size != min_size:
+                musicapplet.widgets.ma_constrain_applet_size_minimum (self.__applet, min_size)
+                self.__constrain_size = min_size
+        else:
+            if self.__constrain_size != -1:
+                musicapplet.widgets.ma_constrain_applet_size_clear (self.__applet)
+                self.__constrain_size = -1
+
+
     ##################################################################
     #
     # Playback controls
@@ -837,6 +868,7 @@ class NotifyPopup (gobject.GObject):
         self.__key = conf.resolve_key ("show_notification")
         self.__handlers = []
         self.__show_source = None
+        self.__have_art = False
 
         conf.client.notify_add (self.__key, self.__notification_config_changed_cb)
 
@@ -848,20 +880,22 @@ class NotifyPopup (gobject.GObject):
 
         if self.__plugin is not None:
             self.__handlers = [
-                self.__plugin.connect ("notify::title", lambda plugin, pspec: self.__queue_show ()),
-                self.__plugin.connect ("notify::artist", lambda plugin, pspec: self.__queue_show ()),
-                self.__plugin.connect ("notify::album", lambda plugin, pspec: self.__queue_show ()),
-                self.__plugin.connect ("notify::art", lambda plugin, pspec: self.__queue_show ()),
+                self.__plugin.connect ("notify::title", lambda plugin, pspec: self.__new_popup ()),
+                self.__plugin.connect ("notify::artist", lambda plugin, pspec: self.__update_popup_text ()),
+                self.__plugin.connect ("notify::album", lambda plugin, pspec: self.__update_popup_text ()),
+                self.__plugin.connect ("notify::art", lambda plugin, pspec: self.__update_popup_art ()),
             ]
         else:
             self.__handlers = []
 
-        self.__show ()
+        self.__new_popup ()
+        self.__update_popup_text ()
+        self.__update_popup_art ()
 
 
-    def __show (self):
+    def __new_popup (self):
         """
-        Show the notification popup, as long as there's a current song.
+        Create and show a new notification popup after a song change.
         """
 
         import pynotify
@@ -870,36 +904,53 @@ class NotifyPopup (gobject.GObject):
             self.__notify.close ()
 
         if self.__plugin is not None and self.__plugin.title is not None and self.__conf.client.get_bool (self.__key):
-            body = []
-            if self.__plugin.artist is not None:
-                body.append (_("<i>by</i> %s") % saxutils.escape (self.__plugin.artist))
-            if self.__plugin.album is not None:
-                body.append (_("<i>from</i> %s") % saxutils.escape (self.__plugin.album))
-
-            self.__notify = pynotify.Notification (saxutils.escape (self.__plugin.title), "\n".join (body), None, self.__applet)
+            # title is automatically escaped, but body is not
+            self.__notify = pynotify.Notification (self.__plugin.title,
+                                                   "",
+                                                   None,
+                                                   self.__applet)
+            self.__have_art = False
             self.__notify.set_urgency (pynotify.URGENCY_LOW)
-            if self.__plugin.art is not None:
-                self.__notify.set_icon_from_pixbuf (scale_pixbuf (self.__plugin.art))
+            self.__update_popup_text ()
+
+            # Don't automatically update the artwork, since there may not be
+            # any, since the title is usually the first to change and libnotify
+            # doesn't allow for removing artwork once it's been set.
 
             self.__notify.show ()
         else:
             self.__notify = None
 
-        self.__show_source = None
-        return False
 
-
-    def __queue_show (self):
+    def __update_popup_text (self):
         """
-        Queue a display of the popup in the very near future.
-
-        This is used to prevent repeatedly creating and destroying the popup
-        as the plugin populates its metadata properties individually, or at
-        least make it much less likely.
+        Update the body text of the existing popup.
         """
 
-        if self.__show_source is None:
-            self.__show_source = gobject.timeout_add (500, self.__show)
+        if self.__notify is not None:
+            # title is automatically escaped, but body is not
+            body = []
+            if self.__plugin.artist is not None:
+                body.append (_("<i>by</i> %s") % saxutils.escape (self.__plugin.artist))
+            if self.__plugin.album is not None:
+                body.append (_("<i>from</i> %s") % saxutils.escape (self.__plugin.album))
+            self.__notify.update (self.__plugin.title,
+                                  "\n".join (body))
+            self.__notify.show ()
+
+
+    def __update_popup_art (self):
+        """
+        Update the art displayed in the existing popup.
+        """
+
+        # In case this is called multiple times with art, ignore all but the
+        # first, since we can't update it anyway.
+
+        if self.__notify is not None and self.__plugin.art is not None and not self.__have_art:
+            self.__have_art = True
+            self.__notify.set_icon_from_pixbuf (scale_pixbuf (self.__plugin.art))
+            self.__notify.show ()
 
 
     def __notification_config_changed_cb (self, client, id, entry, unused):
