@@ -1,5 +1,5 @@
 # Music Applet
-# Copyright (C) 2007 Paul Kuliniewicz <paul@kuliniewicz.org>
+# Copyright (C) 2007-2008 Paul Kuliniewicz <paul@kuliniewicz.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02111-1301, USA.
 
 import gobject
-from gtk import gdk
 
 import musicapplet.defs
 import musicapplet.player
@@ -27,7 +26,7 @@ INFO = {
         "version": musicapplet.defs.VERSION,
         "icon-name": "rhythmbox",
         "author": "Paul Kuliniewicz <paul@kuliniewicz.org>",
-        "copyright": "(C) 2007 Paul Kuliniewicz",
+        "copyright": "(C) 2007-2008 Paul Kuliniewicz",
         "website": "http://www.kuliniewicz.org/music-applet/",
 }
 
@@ -58,6 +57,8 @@ class RhythmboxPlugin (musicapplet.player.DBusPlugin):
         self.__player = None
         self.__shell = None
 
+        self.__art_source = None
+
 
     ##################################################################
     #
@@ -99,50 +100,6 @@ class RhythmboxPlugin (musicapplet.player.DBusPlugin):
             self._set_no_song ()
 
 
-    ##################################################################
-    #
-    # Title
-    #
-    ##################################################################
-
-
-    def __set_title (self, title):
-        self.title = title
-
-
-    ##################################################################
-    #
-    # Artist
-    #
-    ##################################################################
-
-
-    def __set_artist (self, artist):
-        self.artist = artist
-
-
-    ##################################################################
-    #
-    # Album
-    #
-    ##################################################################
-
-
-    def __set_album (self, album):
-        self.album = album
-
-
-    ##################################################################
-    #
-    # Duration
-    #
-    ##################################################################
-
-
-    def __set_duration (self, duration):
-        self.duration = duration
-
-
     ##################################################################
     #
     # Playing
@@ -165,6 +122,10 @@ class RhythmboxPlugin (musicapplet.player.DBusPlugin):
 
 
     def __set_uri (self, uri):
+        if self.__art_source is not None:
+            gobject.source_remove (self.__art_source)
+            self.__art_source = None
+
         if uri is not None and uri != "":
             self.__uri = uri
             self.__shell.getSongProperties (uri,
@@ -175,30 +136,66 @@ class RhythmboxPlugin (musicapplet.player.DBusPlugin):
 
 
     def __set_song_properties (self, properties):
-        self.title = properties.get ("title")
-        self.artist = properties.get ("artist")
-        self.album = properties.get ("album")
+        self.title = properties.get ("title") or None
+        self.artist = properties.get ("artist") or None
+        self.album = properties.get ("album") or None
         self.duration = properties.get ("duration", -1)
         self.rating = properties.get ("rating", -1.0)
-        if properties.has_key ("rb:coverArt-uri"):
-            self.art = gdk.pixbuf_new_from_file (properties.get ("rb:coverArt-uri"))
+        if properties.has_key ("rb:coverArt"):
+            self.art = self._load_art (properties.get ("rb:coverArt"))
+        elif properties.has_key ("rb:coverArt-uri"):
+            self.art = self._load_art (properties.get ("rb:coverArt-uri"))
         else:
             self.art = None
+            # Rhythmbox sometimes takes a few seconds before actually loading
+            # the art, so maybe try again a little later.
+            if self.elapsed < 3 and self.__art_source is None:
+                self.__art_source = gobject.timeout_add (3000, self.__refetch_art)
+
+
+    def __set_song_properties_art_only (self, properties):
+        """
+        See if art is now available for the album, and if so, show it.
+        """
+
+        if properties.has_key ("rb:coverArt"):
+            self.art = self._load_art (properties.get ("rb:coverArt"))
+        elif properties.has_key ("rb:coverArt-uri"):
+            self.art = self._load_art (properties.get ("rb:coverArt-uri"))
+
+
+    def __refetch_art (self):
+        """
+        Look up the song properties again in case art is now available.
+        """
+
+        self.__shell.getSongProperties (self.__uri,
+                                        reply_handler=self.__set_song_properties_art_only,
+                                        error_handler=self._report_dbus_error)
+        return False
 
 
     def __property_changed (self, uri, property, old_value, new_value):
+        # Rhythmbox uses 'title' for the radio station name, but displays
+        # it as though it were 'album'.  Let's do likewise.  Of course,
+        # override if -artist or -album gets updated.
+
         if property == "title":
             self.title = new_value
-        elif property == "artist":
-            self.artist = new_value
-        elif property == "album":
-            self.album = new_value
+        elif property == "rb:stream-song-title":
+            if self.album is None:
+                self.album = self.title
+            self.title = new_value or None
+        elif property == "artist" or property == "rb:stream-song-artist":
+            self.artist = new_value or None
+        elif property == "album" or property == "rb:stream-song-album":
+            self.album = new_value or None
         elif property == "duration":
             self.duration = new_value
         elif property == "rating":
             self.rating = new_value
-        elif property == "rb:coverArt-uri":
-            self.art = gdk.pixbuf_new_from_file (new_value)
+        elif property == "rb:coverArt" or property == "rb:coverArt-uri":
+            self._load_art (new_value)
 
 
     ##################################################################
