@@ -18,7 +18,7 @@
 from gettext import gettext as _
 import gobject
 import gtk
-from gtk import gdk, glade
+from gtk import glade
 
 import os
 
@@ -208,6 +208,9 @@ class XMMS2Plugin (musicapplet.player.PluginBase):
         if "album" in info:
             if self.album != info["album"]:
                 self.album = info["album"]
+        elif "channel" in info:
+            if self.album != info["channel"]:
+                self.album = info["channel"]
         else:
             self.album = None
 
@@ -228,12 +231,7 @@ class XMMS2Plugin (musicapplet.player.PluginBase):
         art_url = self.__find_art_url (info)
         if self.__art_url != art_url:
             self.__art_url = art_url
-            if art_url is not None:
-                if art_url.startswith ("file://"):
-                    self.art = gdk.pixbuf_new_from_file (art_url[7:])
-                # TODO: A way to pull it from http:// or whatever else might be used
-            else:
-                self.art = None
+            self._load_art (art_url)
 
 
     def __find_art_url (self, info):
